/*
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:    lo1.cs

   Purpose: Demonstrates how to solve small linear
            optimization problem using the MOSEK C# API.
 */


using System;

class msgclass : mosek.Stream 
{
  string prefix;
  public msgclass (string prfx) 
  {
    prefix = prfx;
  }
  
  public override void streamCB (string msg)
  {
    Console.Write ("{0}{1}", prefix,msg);
  }
}

public class lo1
{  
    public static void Main ()
    {
        const int NUMCON = 3;
        const int NUMVAR = 4;
        const int NUMANZ = 9;

        // Since the value infinity is never used, we define
        // 'infinity' symbolic purposes only
        double
            infinity = 0;

        double[] c    = {3.0, 1.0, 5.0, 1.0};
        int[]    ptrb = {0, 2, 5, 7};
        int[]    ptre = {2, 5, 7, 9};
        int[]    asub = { 0, 1,
                          0, 1, 2,
                          0, 1,
                          1, 2};
        double[] aval = { 3.0, 2.0,
                          1.0, 1.0, 2.0,
                          2.0, 3.0,
                          1.0, 3.0};
        mosek.boundkey[] bkc  = {mosek.boundkey.fx,
                                 mosek.boundkey.lo,
                                 mosek.boundkey.up};
        double[] blc  = {30.0,
                          15.0,
                          -infinity};
        double[] buc  = {30.0,
                          +infinity,
                          25.0};
        
        mosek.boundkey[]  bkx  = {mosek.boundkey.lo,
                          mosek.boundkey.ra,
                          mosek.boundkey.lo,
                          mosek.boundkey.lo};
        double[]  blx  = {0.0,
                          0.0,
                          0.0,
                          0.0};
        double[]  bux  = {+infinity,
                          10.0,
                          +infinity,
                          +infinity};
        mosek.Task 
            task = null;
        mosek.Env  
            env  = null;

        double[] xx  = new double[NUMVAR];

        try
          {
            // Make mosek environment. 
            env  = new mosek.Env ();
            // Direct the env log stream to the user specified
            // method env_msg_obj.streamCB 
            env.set_streamCB (mosek.streamtype.log, new msgclass ("[env]"));
            // Initialize the environment.
            env.init ();
            // Create a task object linked with the environment env.
            task = new mosek.Task (env, NUMCON,NUMVAR);
            // Directs the log task stream to the user specified
            // method task_msg_obj.streamCB
            task.set_streamCB (mosek.streamtype.log, new msgclass ("[task]"));
            task.inputdata(NUMCON,NUMVAR,
                           c,
                           0.0,
                           ptrb,
                           ptre,
                           asub,
                           aval,
                           bkc,
                           blc,
                           buc,
                           bkx,
                           blx,
                           bux);
            task.putobjsense(mosek.objsense.maximize);
            try
              {
                task.optimize();
              }
            catch (mosek.Warning w)
              {
                Console.WriteLine("Mosek warning:");
                Console.WriteLine (w.Code);
                Console.WriteLine (w);
              }
            task.getsolutionslice(mosek.soltype.bas, /* Basic solution.       */
                                  mosek.solitem.xx,  /* Which part of solution.  */
                                  0,                 /* Index of first variable. */
                                  NUMVAR,            /* Index of last variable+1 */
                                  xx);

            for(int j = 0; j < NUMVAR; ++j)
              Console.WriteLine ("x[{0}]:{1}", j,xx[j]);
          }
        catch (mosek.Exception e)
            {
                Console.WriteLine (e.Code);
                Console.WriteLine (e);
            }

        if (task != null) task.Dispose ();
        if (env  != null)  env.Dispose ();
    }
}
