package qcqo1;

/* 
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:      qcqo1.java

   Purpose:   Demonstrate how to solve a quadratic
              optimization problem using the MOSEK API.

              minimize  x0^2 + 0.1 x1^2 +  x2^2 - x0 x2 - x1 
              s.t 1 <=  x0 + x1 + x2 - x0^2 - x1^2 - 0.1 x2^2 + 0.2 x0 x2 
              x >= 0              
                       
 */



import mosek.*;

public class qcqo1
{
    static final int NUMCON = 1;   /* Number of constraints.             */
    static final int NUMVAR = 3;   /* Number of variables.               */
    static final int NUMANZ = 3;   /* Number of numzeros in A.           */
    static final int NUMQNZ = 4;   /* Number of nonzeros in Q.           */
    public static void main (String[] args)
    {
        // Since the value infinity is never used, we define
        // 'infinity' symbolic purposes only
        double infinity = 0;
        double[] c = {0.0, -1.0, 0.0};

        int[]    bkc  = {mosek.Env.boundkey.lo};
        double[] blc = {1.0};
        double[] buc = {infinity};
        
        int[]  bkx   = {mosek.Env.boundkey.lo,
                        mosek.Env.boundkey.lo,
                        mosek.Env.boundkey.lo};        
        double[] blx = {0.0,
                        0.0,
                        0.0};
        double[] bux = {infinity,
                        infinity,
                        infinity};
        
        int[] ptrb = {0, 1, 2 };
        int[] ptre = {1, 2, 3};
        int[]    asub  = { 0,   0,   0 };
        double[] aval  = { 1.0, 1.0, 1.0 };

        double[] xx   = new double[NUMVAR];
  
        mosek.Env  
            env;
        mosek.Task 
            task;
 
        env  = new mosek.Env ();
        try
            {
                env.init ();
                
                task = new mosek.Task (env, NUMCON,NUMVAR);
        
//REF:func:task:inputdata
                task.inputdata(NUMCON,NUMVAR,
                               c,0.0,
                               ptrb,
                               ptre,
                               asub,
                               aval,
                               bkc,
                               blc,
                               buc,
                               bkx,
                               blx,
                               bux);
                /*
                 * The lower triangular part of the Q
                 * matrix in the objective is specified.
                 */

                int[]   qosubi = { 0,   1,   2,    2 };
                int[]   qosubj = { 0,   1,   0,    2 };
                double[] qoval = { 2.0, 0.2, -1.0, 2.0 };
            
                /* Input the Q for the objective. */

                task.putqobj(qosubi,qosubj,qoval);

                /*
                 * The lower triangular part of the Q^0
                 * matrix in the first constraint is specified.
                 This corresponds to adding the term
                 - x0^2 - x1^2 - 0.1 x2^2 + 0.2 x0 x2
                */

                int[]    qsubi = {0,   1,    2,   2  };
                int[]    qsubj = {0,   1,    2,   0  };
                double[] qval =  {-2.0, -2.0, -0.2, 0.2};

                /* put Q^0 in constraint with index 0. */
                
//REF:func:task:putqconk
                task.putqconk (0,
                               qsubi, 
                               qsubj, 
                               qval); 

                task.putobjsense(mosek.Env.objsense.min);
                task.optimize();
 
//REF:enum:soltype:itr
//REF:enum:solitem:xx
                task.getsolutionslice(mosek.Env.soltype.itr,
                                      mosek.Env.solitem.xx,
                                      0,
                                      NUMVAR,
                                      xx);
            
                System.out.println ("Primal solution");
                for ( int j=0; j<NUMVAR; ++j )
                    System.out.println("x[" + j + "]: " + xx[j]);

                task.cleanup ();
                env.cleanup  ();
            }
        catch (mosek.ArrayLengthException e)
            /* Catch both Error and Warning */
            {
                System.out.println ("An error/warning was encountered");
                System.out.println (e.toString ());
            }
        catch (mosek.Exception e)
            {
                System.out.println ("An error/warning was encountered");
                System.out.println (e.msg);
            }
    } /* Main */
}
