#
#   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.
#
#   File:      cqo1.py
#
#   Purpose:   Demonstrates how to solve a small conic qaudratic
#              optimization problem using the MOSEK API.
#


import sys
import os

import pymosek as mosek

try:
    from Numeric  import array, Float, zeros,ones
except ImportError:
    from mosekarr import array, Float, zeros,ones


# Since the actual value of Infinity is ignores, we define it solely
# for symbolic purposes:
inf = 0.0

# Define a stream printer to grab output from MOSEK
def streamprinter(text):
    sys.stdout.write(text)
    sys.stdout.flush()

# We might write everything directly as a script, but it looks nicer
# to create a function.
def main ():
    # Create a handle to MOSEK
    mskhandle = mosek.mosek ()
    # Make a MOSEK environment
    env = mskhandle.Env ()
    # Attach a printer to the environment
    env.set_Stream (mosek.streamtype.log, streamprinter)
    # Initialize the environment
    env.init ()

    # Create a task
    task = env.Task(0,0)
    # Attach a printer to the task
    task.set_Stream (mosek.streamtype.log, streamprinter)

    bkc = array([ mosek.boundkey.fx ])
    blc = array([ 1.0 ])
    buc = array([ 1.0 ])

    c   = array([               0.0,              0.0,              0.0,
                                0.0,              1.0,              1.0 ])
    bkx = array([ mosek.boundkey.lo,mosek.boundkey.lo,mosek.boundkey.lo,
                  mosek.boundkey.lo,mosek.boundkey.fr,mosek.boundkey.fr ])
    blx = array([               0.0,              0.0,              0.0,
                                0.0,             -inf,             -inf ])
    bux = array([               inf,              inf,              inf,
                                inf,              inf,              inf ])

    asub = array([ 0,   0,   0,   0   ])
    acof = array([ 1.0, 1.0, 1.0, 1.0 ])

    #      column  0  1  2  3  4  5
    ptrb = array([ 0, 1, 2, 3, 4, 4 ])
    ptre = array([ 1, 2, 3, 4, 4, 4 ])

    numvar = len(bkx)
    numcon = len(bkc)

    # Load data into task
    task.inputdata (numcon,numvar,
                    c,     # linear objective coefficients
                    0.0,   # objective fixed value
                    ptrb,
                    ptre,
                    asub,acof,
                    bkc,blc,buc,
                    bkx,blx,bux)

    # Input the cones
##REF:func:task:appendcone
##REF:enum:conetype:quad
    task.appendcone(mosek.conetype.quad,
                    0.0,
                    array([ 4, 0, 2 ]))
##REF:enum:conetype:rquad
    task.appendcone(mosek.conetype.quad,
                    0.0,
                    array([ 5, 1, 3 ]))

    # Input objective sense
    task.putobjsense(mosek.objsense.minimize)
    

    # Optimize
    task.optimize ()

    # Output the solution
    task.solutionsummary (mosek.streamtype.msg);

    xx = zeros(numvar, Float)
    task.getsolutionslice(mosek.soltype.itr,
                          mosek.solitem.xx,
                          0, numvar,
                          xx)
    print "x =", xx

    return None


# call the main function
try:
    main ()
except mosek.Exception, e:
    print "ERROR: %s" % str(e.errno)
    if e.msg is not None:
        print "\t%s" % e.msg
    sys.exit(1)
except:
    import traceback
    traceback.print_exc()
    sys.exit(1)

